/*
 # ---------------------------------------------------------------------
 #
 # Copyright (c) CREATIS (Centre de Recherche en Acquisition et Traitement de l'Image
 #                        pour la SantÈ)
 # Authors : Eduardo Davila, Frederic Cervenansky, Claire Mouton
 # Previous Authors : Laurent Guigues, Jean-Pierre Roux
 # CreaTools website : www.creatis.insa-lyon.fr/site/fr/creatools_accueil
 #
 #  This software is governed by the CeCILL-B license under French law and
 #  abiding by the rules of distribution of free software. You can  use,
 #  modify and/ or redistribute the software under the terms of the CeCILL-B
 #  license as circulated by CEA, CNRS and INRIA at the following URL
 #  http://www.cecill.info/licences/Licence_CeCILL-B_V1-en.html
 #  or in the file LICENSE.txt.
 #
 #  As a counterpart to the access to the source code and  rights to copy,
 #  modify and redistribute granted by the license, users are provided only
 #  with a limited warranty  and the software's author,  the holder of the
 #  economic rights,  and the successive licensors  have only  limited
 #  liability.
 #
 #  The fact that you are presently reading this means that you have had
 #  knowledge of the CeCILL-B license and that you accept its
 terms.
 # ------------------------------------------------------------------------ */


/*=========================================================================
  Program:   bbtk
  Module:    $RCSfile: bbtkQt.cxx,v $
  Language:  C++
  Date:      $Date: 2012/11/16 08:49:01 $
  Version:   $Revision: 1.4 $
=========================================================================*/


#include "bbtkWt.h"
#include "bbtkMessageManager.h"

#ifdef USE_WT




namespace bbtk
{
  //=========================================================================
  class WtApp;
  Wt::WApplication *createApplication(const Wt::WEnvironment& env);
  //=========================================================================

  //=========================================================================
  static WtApp* mgWtApp = 0;
  static Wt::WContainerWidget* mgTopWindow = 0;
  static int mgNbWindowsAlive  = 0;
  //=========================================================================
  
  //=========================================================================
  // BBTKWXSIG
  static WWt::Signal_type mgWtSignal;
  // \BBTKWXSIG
  //=========================================================================

 
  //=========================================================================
  // The wxApp class which is used when no user wxApp was created
 // The application creates itself completly empty.
// The default values for the port and address are port:8080 address 0.0.0.0
  class WtApp : public Wt::WApplication
  {
  public:
    WtApp(const Wt::WEnvironment& env);
    ~WtApp();
    static int WtWRun();
		//WtWApplication *createApplication(const WtWEnvironment& env);
    
  };

	WtApp::WtApp(const Wt::WEnvironment& env):
		Wt::WApplication(env)
	{
		//root()->addWidget(WWt::GetTopWindow());
		// There is not additional configuration. The new WtApp must have the same behavior that one Wt::WApplication.
		
	}

	WtApp::~WtApp()
	{

	}
/*
	int WtApp::WtWRun()
	{
		char  *argv[7] =  {"./WtTest","--docroot",".","--http-address","0.0.0.0","--http-port","8080"};
		int argc = 7;
		return Wt::WRun(argc,argv,&createApplication);	
	}


	Wt::WApplication *createApplication(const Wt::WEnvironment& env)
	{
		return new WtApp(env);
	}
*/



  //=========================================================================

  //=========================================================================
  void WWt::CreateWtAppIfNeeded()
  {
		std::cout<<"CREATING APP IF NEEDED --------- DELETE ME ------- bbtkWt.cxx"<<std::endl;
		if(Wt::WApplication::instance()==0)
		{
			if (mgWtApp != 0) 
	  	{
	    	bbtkGlobalError("Wt::CreateWtAppIfNeeded() : INTERNAL ERROR ! (Wt::WApplication::instance()==0) && (mgWtApp != 0)");
			}
			bbtkDebugMessage("wt",1,"  --> Creating bbtk WtApp"<<std::endl);  
			//WtApp::WtWRun();
		}		
		bbtkDebugMessage("process",2,"Create WtApp If needed debuging");
  }
  //=========================================================================

  //=========================================================================
  void WWt::DestroyWtAppIfNeeded()
  {
    if (mgWtApp!= 0) 
      {
	//	bbtkDebugMessage("wx",1,"  --> Destructing bbtk WxApp"<<std::endl);
	//delete mgWxApp;
	//	mgWxApp = 0;
	// Uninit wx
	//	wxUninitialize();
      }
  }
  //=========================================================================


  //=========================================================================
  void WWt::ProcessPendingEvents()
  {
    /*
    if (Wx::GetTopWindow() != 0)
      {
	Wx::GetTopWindow()->DestroyChildren();
      }
    */

    /*
    wxApp* a = (wxApp*)wxApp::GetInstance();
    if (a==0) return;
    
    std::cout << "$$$$$$$$$$$$$$ PROCESS PENDING "<<std::endl;
    while (a->Pending())
      a->Dispatch();
    std::cout << "$$$$$$$$$$$$$$ PROCESS PENDING DONE"<<std::endl;
    */
  }


  //=========================================================================
  Wt::WContainerWidget* WWt::GetTopWindow() 
  { 
    return mgTopWindow; 
  }
  //=========================================================================
  

  // BBTKWXSIG
  //=========================================================================
  void WWt::AddSignalObserver(Slot_function_type f)
  {
    mgWtSignal.connect(f);
  }
  //=========================================================================
  // \BBTKWXSIG

  
  //=========================================================================
  void WWt::SetTopWindow(Wt::WContainerWidget* w) 
  {
    if ( mgTopWindow ) 
      {
	bbtkGlobalError("Wt::SetTopWindow : top window already set !");
      } 
    //mgTopWindow = w;
  }
  //=========================================================================


  //=========================================================================
  void WWt::IncNbWindowsAlive() 
  { 
    mgNbWindowsAlive++; 
    bbtkDebugMessage("wt",2,"* Number of windows alive = "<<mgNbWindowsAlive
		     <<std::endl);
    // BBTKWXSIG
    mgWtSignal();
    // \BBTKWXSIG
  }
  //=========================================================================

  //=========================================================================
  void WWt::DecNbWindowsAlive()
  { 
    mgNbWindowsAlive--; 
    bbtkDebugMessage("wt",2,"* Number of windows alive = "<<mgNbWindowsAlive
		     <<std::endl);

    // BBTKWXSIG
    mgWtSignal();
    // \BBTKWXSIG
  }
  //=========================================================================

  //=========================================================================
  int  WWt::GetNbWindowsAlive() 
  { 
    return mgNbWindowsAlive; 
  }
  //=========================================================================
  
  //=========================================================================
  bool WWt::IsSomeWindowAlive() 
  { 
    return (mgNbWindowsAlive!=0);
  }
  //=========================================================================
  

  //=========================================================================
  WWt::BusyCursor::BusyCursor()
  {
   // mCursor = 0;
    if (Wt::WApplication::instance()!=0) 
      { 	
	bbtkDebugMessage("wt",2,
			 "Wt::BusyCursor::BusyCursor() : creating new cursor"
			 <<std::endl);
	//mCursor = new wxBusyCursor; 
    }
  }
  //=========================================================================
  //=========================================================================
  WWt::BusyCursor::~BusyCursor()
  {
   // if (mCursor)
     // { 
	//delete mCursor;
	//bbtkDebugMessage("wt",2,
	//		 "Wt::BusyCursor::~BusyCursor() : deleting cursor"<<std::endl);
      //}
  }
  //=========================================================================



} // namespace bbtk

#else
//=======================================================================
// WITHOUT WX
//=========================================================================
namespace bbtk
{
   //=========================================================================
  WWt::BusyCursor::BusyCursor()
  {
  }
  WWt::BusyCursor::~BusyCursor()
  {
  }
  //=========================================================================

} // namespace bbtk
#endif


