/*
 # ---------------------------------------------------------------------
 #
 # Copyright (c) CREATIS (Centre de Recherche en Acquisition et Traitement de l'Image
 #                        pour la SantÈ)
 # Authors : Eduardo Davila, Frederic Cervenansky, Claire Mouton
 # Previous Authors : Laurent Guigues, Jean-Pierre Roux
 # CreaTools website : www.creatis.insa-lyon.fr/site/fr/creatools_accueil
 #
 #  This software is governed by the CeCILL-B license under French law and
 #  abiding by the rules of distribution of free software. You can  use,
 #  modify and/ or redistribute the software under the terms of the CeCILL-B
 #  license as circulated by CEA, CNRS and INRIA at the following URL
 #  http://www.cecill.info/licences/Licence_CeCILL-B_V1-en.html
 #  or in the file LICENSE.txt.
 #
 #  As a counterpart to the access to the source code and  rights to copy,
 #  modify and redistribute granted by the license, users are provided only
 #  with a limited warranty  and the software's author,  the holder of the
 #  economic rights,  and the successive licensors  have only  limited
 #  liability.
 #
 #  The fact that you are presently reading this means that you have had
 #  knowledge of the CeCILL-B license and that you accept its terms.
 # ------------------------------------------------------------------------ */


/*=========================================================================
  Program:   bbtk
  Module:    $RCSfile: bbvtkSegmentationConnectivity.cxx,v $
  Language:  C++
  Date:      $Date: 2012/11/16 08:51:58 $
  Version:   $Revision: 1.8 $
=========================================================================*/

/**
 *  \file 
 *  \brief 
 */

#ifdef _USE_VTK_

#include "bbvtkSegmentationConnectivity.h"
#include "bbvtkPackage.h"

#include "bbtkStaticLecture.h"

namespace bbvtk
{
  BBTK_ADD_BLACK_BOX_TO_PACKAGE(vtk,SegmentationConnectivity);
  BBTK_BLACK_BOX_IMPLEMENTATION(SegmentationConnectivity,
				bbtk::AtomicBlackBox);

// --------------------------------------------------------------

  void SegmentationConnectivity::bbUserSetDefaultValues() 
  { 
	  thresh2	= NULL;
//	  cast2		= NULL;
	  connect2	= NULL;
	  cast4		= NULL;
	  bbSetInputIn(NULL);
	  std::vector<int> position;
	  position.push_back(0);
	  position.push_back(0);
	  position.push_back(0);
	  bbSetInputPositionXYZ(position);
	  std::vector<int> threshold;
	  threshold.push_back(0);
	  threshold.push_back(1);
	  bbSetInputThresholdMinMax(threshold);
	  bbSetOutputOut(NULL);
	  bbSetInputOutputFormat("SAME");
	  bbSetInputValue(255);
  }

// --------------------------------------------------------------	
	void SegmentationConnectivity::bbUserInitializeProcessing()
	{
		thresh2 = vtkImageThreshold::New();
		thresh2->SetInValue(255);
		thresh2->SetOutValue(0);
		thresh2->SetOutputScalarTypeToUnsignedChar();
		connect2 = vtkImageSeedConnectivity::New();
//EED 2017-01-01 Migration VTK7
#if VTK_MAJOR_VERSION <= 5
		connect2->SetInput(thresh2->GetOutput());
#else
		connect2->SetInputData(thresh2->GetOutput());
#endif
		connect2->SetInputConnectValue(255);
		connect2->SetOutputConnectedValue( 255 );
		connect2->SetOutputUnconnectedValue(0);
		cast4 = vtkImageCast::New();
//EED 2017-01-01 Migration VTK7
#if VTK_MAJOR_VERSION <= 5
		cast4->SetInput(connect2->GetOutput());
#else
		cast4->SetInputData(connect2->GetOutput());
#endif
	}

// --------------------------------------------------------------		
  void SegmentationConnectivity::bbUserFinalizeProcessing() 
  { 
	  if (thresh2!=NULL)
	  {
		  thresh2->Delete();
		  thresh2=NULL;
	  }
//	  if (cast2!=NULL)
//	  {
//		  cast2->Delete();
//		  cast2=NULL;
//	  }
	  if (connect2!=NULL)
	  {
		  connect2->Delete();
		  connect2=NULL;
	  }
	  if (cast4!=NULL)
	  {
		  cast4->Delete();
		  cast4=NULL;
	  }  
  }

// --------------------------------------------------------------
  void SegmentationConnectivity::DoProcess()
  {
	int x,y,z;
	int i,numPoints = bbGetInputPositionXYZ().size()/3;
	int verif = (bbGetInputPositionXYZ().size()-1) % 3;
	if (( numPoints>=1) && (verif==2) && (bbGetInputIn()!=NULL)) 
	{
//EED 2020-01-18
		int outputformat = bbtkStaticLecture::GetTypeFormat( bbGetInputOutputFormat() , bbGetInputIn() );
//        int outputformat=4;
		vtkImageData *imagedata = bbGetInputIn();
//EED 2017-01-01 Migration VTK7
#if VTK_MAJOR_VERSION <= 5
		imagedata->UpdateInformation();
		imagedata->SetUpdateExtent(imagedata->GetWholeExtent());
		imagedata->Update();
		thresh2->SetInput(imagedata);
#else
//		imagedata->SetExtent(imagedata->GetExtent());
		thresh2->SetInputData(imagedata);
#endif
		thresh2->ThresholdBetween(bbGetInputThresholdMinMax()[0], bbGetInputThresholdMinMax()[1]);
		thresh2->Update();
		connect2->RemoveAllSeeds ();
		int ext[6];
//EED 2017-01-01 Migration VTK7
#if VTK_MAJOR_VERSION <= 5
		imagedata->GetWholeExtent(ext);
#else
		imagedata->GetExtent(ext);
#endif
		int maxX = ext[1]-ext[0]+1;
		int maxY = ext[3]-ext[2]+1;
		int maxZ = ext[5]-ext[4]+1;
		for (i=0;i<numPoints;i++)
		{
			x = bbGetInputPositionXYZ()[i*3+0];
			y = bbGetInputPositionXYZ()[i*3+1];
			z = bbGetInputPositionXYZ()[i*3+2];
			if ( (x>=0) || (x<maxX) || (y>=0) || (y<maxY) || (z>=0) || (z<maxZ) )
			{
	  			connect2->AddSeed( x,y,z );
			} else {
				printf("EED WARNNIG! In box: SegmentationConnectivity points list out of range \n");
			} // if point inside image 
		} // for i
		connect2->SetOutputConnectedValue( bbGetInputValue() );
		cast4->SetOutputScalarType( outputformat );
		connect2->Modified();
		connect2->Update();
		cast4->Update();
    bbSetOutputOut(cast4->GetOutput() );
	}  else {
		printf("EED WARNNIG! In box: SegmentationConnectivity List of points are not coherent. \n");
		bbSetOutputOut( NULL );
	}// if numPoints verif
}
  
}// EO namespace bbvtk

#endif //_USE_VTK_





